@@ -1,4 +1,4 @@
-/* $Id: gweather-dialog.c,v 1.12 2002/01/23 16:07:49 kevinv Exp $ */
+/* $Id: gweather-dialog.c,v 1.15 2002/03/01 04:53:15 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -74,7 +74,7 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
 
   gw_applet->gweather_dialog = gtk_dialog_new_with_buttons (_("Forecast"), NULL,
 						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 						  NULL);
 
   if (gw_applet->gweather_pref.radar_enabled)
@@ -307,7 +307,6 @@ g_print ("draw radar \n");
 
 
       /* PUSH */
-      gtk_widget_push_visual (gdk_rgb_get_visual ());
       gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
       ebox = gtk_event_box_new ();
@@ -321,7 +320,6 @@ g_print ("draw radar \n");
 
       /* POP */
       gtk_widget_pop_colormap ();
-      gtk_widget_pop_visual ();
 
 
       radar_link_alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
@@ -373,7 +371,7 @@ void gweather_dialog_display_toggle (GWeatherApplet *gw_applet)
 void gweather_dialog_update (GWeatherApplet *gw_applet)
 {
     const gchar *forecast;
-    GdkFont* detailed_forecast_font = gdk_fontset_load ( "fixed" );
+    /*GdkFont* detailed_forecast_font = gdk_fontset_load ( "fixed" );*/
 
     /* Check for parallel network update in progress */
     if(gw_applet->gweather_info == NULL)
@@ -403,12 +401,12 @@ void gweather_dialog_update (GWeatherApplet *gw_applet)
     gtk_text_forward_delete(GTK_TEXT(gw_applet->forecast_text), gtk_text_get_length(GTK_TEXT(gw_applet->forecast_text)));
     forecast = weather_info_get_forecast(gw_applet->gweather_info);
     if (forecast) {
-        gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, forecast, strlen(forecast));
+        gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), NULL, NULL, NULL, forecast, strlen(forecast));
     } else {
         if (gw_applet->gweather_pref.detailed)
-            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, _("Detailed forecast not available for this location.\nPlease try the state forecast; note that IWIN forecasts are available only for US cities."), -1);
+            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), NULL, NULL, NULL, _("Detailed forecast not available for this location.\nPlease try the state forecast; note that IWIN forecasts are available only for US cities."), -1);
         else
-            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), detailed_forecast_font, NULL, NULL, _("State forecast not available for this location.\nPlease try the detailed forecast; note that IWIN forecasts are available only for US cities."), -1);
+            gtk_text_insert(GTK_TEXT(gw_applet->forecast_text), NULL, NULL, NULL, _("State forecast not available for this location.\nPlease try the detailed forecast; note that IWIN forecasts are available only for US cities."), -1);
     }
 
     /* Update radar map */
