@@ -1,4 +1,4 @@
-/* $Id: weather.c,v 1.33 2002/02/10 17:20:36 kevinv Exp $ */
+/* $Id: weather.c,v 1.42 2002/03/01 04:53:15 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -52,16 +52,16 @@ static gchar *weather_proxy_passwd = NULL;
 
 #define TEMP_F_TO_C(f)  (((f) - 32.0) * 0.555556)
 #define TEMP_C_TO_F(c)  (((c) * 1.8) + 32.0)
-#define TEMP_UNIT_STR(units)  (((units) == UNITS_IMPERIAL) ? "\260F" : "\260C")
+#define TEMP_UNIT_STR(units)  (((units) == UNITS_IMPERIAL) ? "\302\260 F" : "\302\260 C")
 
 #define WINDSPEED_KNOTS_TO_KPH(knots)  ((knots) * 1.851965)
 #define WINDSPEED_KPH_TO_KNOTS(kph)    ((kph) * 0.539967)
 #define WINDSPEED_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("knots") : _("kph"))
 
-#define PRESSURE_INCH_TO_MM(inch)   ((inch) * 25.4)
-#define PRESSURE_MM_TO_INCH(mm)     ((mm) * 0.03937)
+#define PRESSURE_INCH_TO_HPA(inch)   ((inch) * 33.86)
+#define PRESSURE_HPA_TO_INCH(mm)     ((mm) / 33.86)
 #define PRESSURE_MBAR_TO_INCH(mbar) ((mbar) * 0.02963742)
-#define PRESSURE_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("inHg") : _("mmHg"))
+#define PRESSURE_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("inHg") : _("hPa"))
 
 #define VISIBILITY_SM_TO_KM(sm)  ((sm) * 1.609344)
 #define VISIBILITY_KM_TO_SM(km)  ((km) * 0.621371)
@@ -73,14 +73,10 @@ WeatherLocation *weather_location_new (const gchar *name, const gchar *code, con
     WeatherLocation *location;
 
     location = g_new(WeatherLocation, 1);
-    strncpy(location->name, name, WEATHER_LOCATION_NAME_MAX_LEN);
-    location->name[WEATHER_LOCATION_NAME_MAX_LEN] = '\0';
-    strncpy(location->code, code, WEATHER_LOCATION_CODE_LEN);
-    location->code[WEATHER_LOCATION_CODE_LEN] = '\0';
-    strncpy(location->zone, zone, WEATHER_LOCATION_ZONE_LEN);
-    location->zone[WEATHER_LOCATION_ZONE_LEN] = '\0';
-    strncpy(location->radar, radar, WEATHER_LOCATION_RADAR_LEN);
-    location->radar[WEATHER_LOCATION_RADAR_LEN] = '\0';
+    location->name = g_strdup(name);
+    location->code = g_strdup(code);
+    location->zone = g_strdup(zone);
+    location->radar = g_strdup(radar);
 
     return location;
 }
@@ -219,21 +215,16 @@ const gchar *weather_conditions_string (WeatherConditions cond)
 
 /* Locals turned global to facilitate asynchronous HTTP requests */
 
-static WeatherInfoFunc request_cb = NULL;
-static WeatherInfo *request_info = NULL;
 
-static __inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
+static inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
 {
     if (info->requests_pending)
         return FALSE;
 
     /*g_assert(!metar_handle && !iwin_handle && !wx_handle && !met_handle);*/
-    g_assert(!request_info && !request_cb);
 
     info->requests_pending = TRUE;
-    request_cb = cb;
-    request_info = info;
-	
+    	
     return TRUE;
 }
 
@@ -256,10 +247,8 @@ static inline void requests_done_check (WeatherInfo *info)
         info->requests_pending = FALSE;
         /* Next two lines temporarily here */
         if (weather_units == UNITS_METRIC)
-            weather_info_to_metric(request_info);
-        (*request_cb)(request_info);
-        request_cb = NULL;
-        request_info = NULL;
+            weather_info_to_metric(info);
+        update_finish(info);
     }
 }
 
@@ -782,7 +771,7 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     WeatherLocation *loc;
     gchar *metar, *eoln, *body, *temp;
     gboolean success = FALSE;
-    gchar searchkey[WEATHER_LOCATION_CODE_LEN + 2];
+    gchar *searchkey;
 
     info->forecast = NULL;
     loc = info->location;
@@ -791,7 +780,7 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     body[body_len] = '\0';
 
     if (info->metar_buffer == NULL)
-      info->metar_buffer = g_strdup(body);
+    	info->metar_buffer = g_strdup(body);
     else
     {
 	temp = g_strdup(info->metar_buffer);
@@ -803,9 +792,10 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     if (result == GNOME_VFS_ERROR_EOF)
     {
 
-  	g_snprintf (searchkey, sizeof (searchkey), "\n%s", loc->code);
+        searchkey = g_strdup_printf("\n%s", loc->code);
 
         metar = strstr(info->metar_buffer, searchkey);
+        g_free (searchkey);
         if (metar == NULL) {
             success = FALSE;
         } else {
@@ -819,9 +809,9 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
         }
 
         info->valid = success;
-	}
-	else if (result != GNOME_VFS_OK) {
-	    g_print("%s", gnome_vfs_result_to_string(result));
+    }
+    else if (result != GNOME_VFS_OK) {
+	g_print("%s", gnome_vfs_result_to_string(result));
         g_warning(_("Failed to get METAR data.\n"));
     } else {
 	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, metar_finish_read, info);
@@ -830,7 +820,7 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     }
     
     request_done(info->metar_handle, info);
-    
+    g_free (buffer);
     return;
 }
 
@@ -842,12 +832,11 @@ static void metar_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
     int body_len;
     gchar *metar, *eoln;
     gboolean success = FALSE;
-    gchar searchkey[WEATHER_LOCATION_CODE_LEN + 2];
 
     g_return_if_fail(handle == info->metar_handle);
 
     g_return_if_fail(info != NULL);
-    
+   
     body = g_malloc0(DATA_SIZE);
     
     info->metar_buffer = NULL;
@@ -857,6 +846,7 @@ static void metar_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
 	    g_warning (_("WeatherInfo missing location"));
 	    request_done(info->metar_handle, info);
 	    requests_done_check(info);
+	    g_free (body);
 	    return;
     }
 
@@ -864,10 +854,10 @@ static void metar_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
         g_warning(_("Failed to get METAR data.\n"));
         info->metar_handle = NULL;
 	requests_done_check(info); 
+	g_free (body);
     } else {
         gnome_vfs_async_read(handle, body, DATA_SIZE - 1, metar_finish_read, info);
-	}
-
+    }
     return;
 }
 
@@ -1054,7 +1044,7 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     }
     
     request_done(info->iwin_handle, info);
-    
+   
     return;
 }
 
@@ -1070,11 +1060,11 @@ static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result
 	
     g_return_if_fail(info != NULL);
 
-	body = g_malloc0(DATA_SIZE);
+    body = g_malloc0(DATA_SIZE);
 
-	info->iwin_buffer = NULL;	
+    info->iwin_buffer = NULL;	
     if (info->forecast)
-        g_free (info->forecast);
+    g_free (info->forecast);
     info->forecast = NULL;
     loc = info->location;
     if (loc == NULL) {
@@ -1082,18 +1072,20 @@ static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result
 	    request_done(info->iwin_handle, info);
 	    info->iwin_handle = NULL;
 	    requests_done_check(info);
+	    g_free (body);
 	    return;
     }
 
     if (result != GNOME_VFS_OK) {
-        g_warning(_("Failed to get IWIN forecast data.\n"));
+        /* forecast data is not really interesting anyway ;)
+	 * g_warning(_("Failed to get IWIN forecast data.\n")); */
         info->iwin_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
         gnome_vfs_async_read(handle, body, DATA_SIZE - 1, iwin_finish_read, info);
-	}
-	
-	return;
+    }
+    return;
 }
 
 static char *met_reprocess(char *x, int len)
@@ -1257,45 +1249,44 @@ static void met_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 			    gpointer buffer, GnomeVFSFileSize requested, 
 			    GnomeVFSFileSize body_len, gpointer data)
 {
-	WeatherInfo *info = (WeatherInfo *)data;
+    WeatherInfo *info = (WeatherInfo *)data;
     WeatherLocation *loc;
     gchar *body, *forecast, *temp;
-g_print ("read met \n");
-	info->forecast = NULL;
+
+    info->forecast = NULL;
     loc = info->location;
-	body = (gchar *)buffer;
-	body[body_len] = '\0';
+    body = (gchar *)buffer;
+    body[body_len] = '\0';
 	
-	if (info->met_buffer == NULL)
-		info->met_buffer = g_strdup(body);
-	else
-	{
-		temp = g_strdup(info->met_buffer);
-		g_free(info->met_buffer);
-		info->met_buffer = g_strdup_printf("%s%s", temp, body);
-		g_free(temp);
-	}
+    if (info->met_buffer == NULL)
+        info->met_buffer = g_strdup(body);
+    else
+    {
+        temp = g_strdup(info->met_buffer);
+	g_free(info->met_buffer);
+	info->met_buffer = g_strdup_printf("%s%s", temp, body);
+	g_free(temp);
+    }
 	
-	if (result == GNOME_VFS_ERROR_EOF)
-	{
-		g_print ("%s \n", info->met_buffer);
-		forecast = met_parse(info->met_buffer, loc);
+    if (result == GNOME_VFS_ERROR_EOF)
+    {
+	forecast = met_parse(info->met_buffer, loc);
         info->forecast = forecast;
     }
-	else if (result != GNOME_VFS_OK) {
-		g_print("%s", gnome_vfs_result_to_string(result));
-		info->met_handle = NULL;
-		requests_done_check (info);
+    else if (result != GNOME_VFS_OK) {
+	g_print("%s", gnome_vfs_result_to_string(result));
+	info->met_handle = NULL;
+	requests_done_check (info);
         g_warning(_("Failed to get Met Office data.\n"));
     } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
+	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
 
-		return;
+	return;
     }
     
     request_done(info->met_handle, info);
-    
-	return;
+    g_free (buffer);
+    return;
 }
 
 static void met_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
@@ -1309,10 +1300,10 @@ static void met_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     g_return_if_fail(handle == info->met_handle);
 
     g_return_if_fail(info != NULL);
-g_print ("finish open \n");
-	body = g_malloc0(DATA_SIZE);
 
-	info->met_buffer = NULL;	
+    body = g_malloc0(DATA_SIZE);
+
+    info->met_buffer = NULL;	
     info->forecast = NULL;
     loc = info->location;
     g_return_if_fail(loc != NULL);
@@ -1321,11 +1312,11 @@ g_print ("finish open \n");
         g_warning(_("Failed to get Met Office forecast data.\n"));
         info->met_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
-	}
-
-	return;
+    	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
+    }
+    return;
 }
 
 static void metoffice_start_open (WeatherInfo *info)
@@ -1335,7 +1326,7 @@ static void metoffice_start_open (WeatherInfo *info)
     loc = info->location;
   
     url = g_strdup_printf("http://www.metoffice.gov.uk/datafiles/%s.html", loc->zone+1);
-g_print ("%s \n", url);
+
     gnome_vfs_async_open(&info->met_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, met_finish_open, info);
     g_free(url);
@@ -1346,7 +1337,6 @@ g_print ("%s \n", url);
 /* Get forecast into newly alloc'ed string */
 static void iwin_start_open (WeatherInfo *info)
 {
-    gchar state[WEATHER_LOCATION_ZONE_LEN];
     gchar *url;
     WeatherLocation *loc;
 
@@ -1364,88 +1354,64 @@ static void iwin_start_open (WeatherInfo *info)
     	return;
     }
 
-    strncpy(state, loc->zone, 2);
-    state[2] = 0;
     if (weather_forecast == FORECAST_ZONE)
-        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/zone.html", state);
+        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/zone.html",
+			loc->zone);
     else
-        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/state.html", state);
+        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/state.html",
+			loc->zone);
     gnome_vfs_async_open(&info->iwin_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, iwin_finish_open, info);
     g_free(url);
 
 }
 
-static GdkPixmap *wx_construct (gpointer data, gint data_len)
-{
-    GdkPixmap *pixmap;
-    GdkPixbuf *pixbuf;
-    GdkPixbufLoader *pixbuf_loader;
-
-    pixbuf_loader = gdk_pixbuf_loader_new ();
-
-    gdk_pixbuf_loader_write (pixbuf_loader, data, data_len, NULL);
-    gdk_pixbuf_loader_close (pixbuf_loader, NULL);
-
-    pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
-
-    pixmap = NULL;
-
-    if (pixbuf != NULL) {
-	    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, NULL, 127);
-	    g_print ("pixbud != NULL\n");
-    }
-
-    g_object_unref (G_OBJECT (pixbuf_loader));
-
-    return pixmap;
-}
-
 static void wx_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result, 
 			   gpointer buffer, GnomeVFSFileSize requested, 
 			   GnomeVFSFileSize body_len, gpointer data)
 {
     WeatherInfo *info = (WeatherInfo *)data;
     WeatherLocation *loc;
-    gchar *body, *temp;
     GdkPixmap *pixmap = NULL;
 	
-	info->forecast = NULL;
-	info->radar = NULL;
+    info->forecast = NULL;
+    info->radar = NULL;
     loc = info->location;
-	body = (gchar *)buffer;
-	body[body_len] = '\0';
 
-	if (info->radar_buffer == NULL)
-		info->radar_buffer = g_strdup(body);
-	else
-	{
-		temp = g_strdup(info->radar_buffer);
-		g_free(info->radar_buffer);
-		info->radar_buffer = g_strdup_printf("%s%s", temp, body);
-		g_free(temp);
-	}
-	
-	if (result == GNOME_VFS_ERROR_EOF)
-	{
-		pixmap = wx_construct (info->radar_buffer, strlen (info->radar_buffer));
-g_print ("got radar \n");
-        info->radar = pixmap;
-	}
-	else if (result != GNOME_VFS_OK) {
-		g_print("%s", gnome_vfs_result_to_string(result));
+    if (result == GNOME_VFS_OK && body_len != 0) {
+        GError *error = NULL;
+    	gdk_pixbuf_loader_write (info->radar_loader, buffer, body_len, &error);
+        if (error) {
+            g_print ("%s \n", error->message);
+            g_error_free (error);
+        }
+        gnome_vfs_async_read(handle, buffer, DATA_SIZE - 1, wx_finish_read, info);
+        return;
+    }
+    else if (result == GNOME_VFS_ERROR_EOF)
+    {
+        GdkPixbuf *pixbuf;
+    	
+        gdk_pixbuf_loader_close (info->radar_loader, NULL);
+        pixbuf = gdk_pixbuf_loader_get_pixbuf (info->radar_loader);
+
+        if (pixbuf != NULL) {
+            if (info->radar)
+                g_object_unref (info->radar);
+	    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &info->radar, NULL, 127);
+        }
+    }
+    else
+    {
+        g_print("%s", gnome_vfs_result_to_string(result));
         g_warning(_("Failed to get METAR data.\n"));
         info->wx_handle = NULL;
         requests_done_check (info);
-    } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, wx_finish_read, info);
-
-		return;
     }
-
-	request_done(info->wx_handle, info);
-	    
-	return;
+    
+    request_done(info->wx_handle, info);
+    g_free (buffer);    
+    return;
 }
 
 static void wx_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
@@ -1471,8 +1437,10 @@ static void wx_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
         g_warning(_("Failed to get radar map image.\n"));
         info->wx_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
-    	gnome_vfs_async_read(handle, body, DATA_SIZE -1, wx_finish_read, info);
+        gnome_vfs_async_read(handle, body, DATA_SIZE -1, wx_finish_read, info); 
+    	
     }
      return;
 }
@@ -1482,25 +1450,30 @@ static void wx_start_open (WeatherInfo *info)
 {
     gchar *url;
     WeatherLocation *loc;
-	
+    
     g_return_if_fail(info != NULL);
     info->radar = NULL;
+    info->radar_loader = gdk_pixbuf_loader_new ();
     loc = info->location;
     g_return_if_fail(loc != NULL);
 
     if (loc->radar[0] == '-')
         return;
-
+#if 0
     url = g_strdup_printf("http://image.weather.com/images/radar/single_site/%sloc_450x284.jpg", loc->radar);
-g_print ("url %s \n", url);
+#else
+    url = g_strdup_printf ("http://image.weather.com/web/radar/us_%s_ultraradar_medium_usen.jpg", loc->radar);
+#endif
+  
     gnome_vfs_async_open(&info->wx_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, wx_finish_open, info);
+    				 
     g_free(url);
 
     return;
 }
 
-gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, WeatherInfoFunc cb)
+gboolean _weather_info_fill (gpointer applet, WeatherInfo *info, WeatherLocation *location, WeatherInfoFunc cb)
 {
     g_return_val_if_fail(((info == NULL) && (location != NULL)) || \
                          ((info != NULL) && (location == NULL)), FALSE);
@@ -1524,10 +1497,8 @@ gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, Weath
 		}
     }
 
-    /* FIX */
-    
+    /* Update in progress */
     if (!requests_init(cb, info)) {
-        g_warning(_("Another update already in progress!\n"));
         return FALSE;
     }
 
@@ -1552,6 +1523,7 @@ gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, Weath
     info->wx_handle = NULL;
     info->met_handle = NULL;
     info->requests_pending = TRUE;
+    info->applet = applet;
 
     metar_start_open(info);
     iwin_start_open(info);
@@ -1626,6 +1598,7 @@ WeatherInfo *weather_info_config_read (PanelApplet *applet)
     info->visibility = 0;
     info->forecast = "None";
     info->radar = NULL;  /* FIX */
+    info->requests_pending = FALSE;
 
     return info;
 }
@@ -1728,7 +1701,7 @@ void weather_info_to_metric (WeatherInfo *info)
     info->temp = TEMP_F_TO_C(info->temp);
     info->dew = TEMP_F_TO_C(info->dew);
     info->windspeed = WINDSPEED_KNOTS_TO_KPH(info->windspeed);
-    info->pressure = PRESSURE_INCH_TO_MM(info->pressure);
+    info->pressure = PRESSURE_INCH_TO_HPA(info->pressure);
     info->visibility = VISIBILITY_SM_TO_KM(info->visibility);
 
     /* Change units flag */
@@ -1746,7 +1719,7 @@ void weather_info_to_imperial (WeatherInfo *info)
     info->temp = TEMP_C_TO_F(info->temp);
     info->dew = TEMP_C_TO_F(info->dew);
     info->windspeed = WINDSPEED_KPH_TO_KNOTS(info->windspeed);
-    info->pressure = PRESSURE_MM_TO_INCH(info->pressure);
+    info->pressure = PRESSURE_HPA_TO_INCH(info->pressure);
     info->visibility = VISIBILITY_KM_TO_SM(info->visibility);
 
     /* Change units flag */
@@ -1883,10 +1856,12 @@ GdkPixmap *weather_info_get_radar (WeatherInfo *info)
 const gchar *weather_info_get_temp_summary (WeatherInfo *info)
 {
     static gchar buf[10];
-    g_return_val_if_fail(info != NULL, NULL);
+    if (!info)
+        return NULL;
     if (!info->valid)
         return "--";
-    g_snprintf(buf, sizeof (buf), "%d\260", (int)(info->temp + 0.5));
+    g_snprintf(buf, sizeof (buf), "%d\302\260", (int)(info->temp + 0.5));
+    
     return buf;
 }
 
