@@ -1,6 +1,10 @@
+#ifndef PREFERENCES_H__
+#define PREFERENCES_H__
+
 #include <sys/types.h>
 #include <regex.h>
 #include <panel-applet.h>
+#include "mini-commander_applet.h"
 
 #define LENGTH_HISTORY_LIST       50
 #define MAX_COMMAND_LENGTH        500
@@ -8,7 +12,7 @@
 #define MAX_NUM_MACRO_PARAMETERS  100
 #define MAX_MACRO_PATTERN_LENGTH  25
 
-typedef struct struct_properties properties;
+
 
 struct struct_properties
 {
@@ -34,16 +38,12 @@ struct struct_properties
     regex_t *macro_regex[MAX_NUM_MACROS];
 };
 
-extern properties prop;
-extern properties prop_tmp;
 
 extern GtkWidget *applet;
 
-void load_session(void);
-
-void save_session(void);
+properties * load_session(MCData *mcdata);
 
 gint save_session_signal(GtkWidget *widget, const char *privcfgpath, const char *globcfgpath);
 
 void properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname);
-
+#endif
